"""This module implements the BioPAX Level 3 object model as
a set of classes with inheritance. At the top of the class
hierarchy is the generic BioPaxObject."""
from .base import *
from .interaction import *
from .physical_entity import *
from .util import *
from .model import *